export default `:host(.alert){padding:0.75rem 1.25rem;border:1px solid transparent;border-radius:0.25rem;}:host(.alert) a{color:rgb(11, 46, 19);font-weight:700;text-decoration:rgb(11, 46, 19);text-decoration-color:rgb(11, 46, 19);text-decoration-line:none;text-decoration-style:solid;text-decoration-thickness:auto;}:host(.alert) a:hover{color:rgb(11, 46, 19);font-weight:700;text-decoration:underline;}:host(.alert-primary){color:var(--primary-text-color);background-color:var(--primary-bg-subtle);border-color:var(--primary-border-subtle);}:host(.alert-primary) hr{border-top-color:#9fcdff;}:host(.alert-secondary){color:var(--secondary-text-color);background-color:var(--secondary-bg-subtle);border-color:var(--secondary-border-subtle);}:host(.alert-secondary) hr{border-top-color:#c8cbcf;}:host(.alert-success){color:var(--success-text-color);background-color:var(--success-bg-subtle);border-color:var(--success-border-subtle);}:host(.alert-success) hr{border-top-color:#b1dfbb;}:host(.alert-info){color:var(--info-text-color);background-color:var(--info-bg-subtle);border-color:var(--info-border-subtle);}:host(.alert-info) hr{border-top-color:#abdde5;}:host(.alert-warning){color:var(--warning-text-color);background-color:var(--warning-bg-subtle);border-color:var(--warning-border-subtle);}:host(.alert-warning) hr{border-top-color:#ffe8a1;}:host(.alert-danger){color:var(--danger-text-color);background-color:var(--danger-bg-subtle);border-color:var(--danger-border-subtle);}:host(.alert-danger) hr{border-top-color:#f1b0b7;}:host(.alert-light){color:var(--light-text-color);background-color:var(--light-bg-subtle);border-color:var(--light-border-subtle);}:host(.alert-light) hr{border-top-color:#ececf6;}:host(.alert-dark){color:var(--dark-text-color);background-color:var(--dark-bg-subtle);border-color:var(--dark-border-subtle);}:host(.alert-dark) hr{border-top-color:#b9bbbe;}:host(.alert-primary) a{color:var(--primary-text-color);}:host(.alert-secondary) a{color:var(--secondary-text-color);}:host(.alert-success) a{color:var(--success-text-color);}:host(.alert-info) a{color:var(--info-text-color);}:host(.alert-warning) a{color:var(--warning-text-color);}:host(.alert-danger) a{color:var(--danger-text-color);}:host(.alert-light) a{color:var(--light-text-color);}:host(.alert-dark) a{color:var(--dark-text-color);}h1,h2,h3,h4,h5,h6{margin-block-start:0;margin-block-end:0.2em;}`
