export const root = "bk-root"
export default `.bk-root .json-formatter-row .json-formatter-string,.bk-root .json-formatter-row .json-formatter-stringifiable{white-space:pre-wrap;}.ql-bubble .ql-editor{border:1px solid #ccc;}`
